@@ -75,35 +75,50 @@ static int netcfg_get_netmask(struct debconfclient *client, struct netcfg_interf
     int ret, ok = 0;
     union inX_addr addr;
     char *ptr;
+    const char *question, *error;
+
+    switch (interface->address_family) {
+    case AF_INET:
+        question="netcfg/get_netmask";
+        error="netcfg/bad_ipv4_address";
+        break;
+    case AF_INET6:
+        question="netcfg/get_ipv6_netmask";
+        error="netcfg/bad_ipv6_address";
+        break;
+    default:
+        question="netcfg/get_netmask";
+        error="netcfg/bad_ipaddress";
+    }
 
     /* Preseed a vaguely sensible looking default netmask if one wasn't
-     * provided.
+     * provided or does not match the address family.
      */
-    debconf_get (client, "netcfg/get_netmask");
+    debconf_get (client, question);
     ptr = strtrim(client->value);
-    if (empty_str(ptr)) {
+    if (empty_str(ptr) || !inet_pton (interface->address_family, ptr, &addr)) {
         if (interface->address_family == AF_INET) {
-            debconf_set(client, "netcfg/get_netmask", "255.255.255.0");
+            debconf_set(client, question, "255.255.255.0");
         } else if (interface->address_family == AF_INET6) {
-            debconf_set(client, "netcfg/get_netmask", "ffff:ffff:ffff:ffff::");
+            debconf_set(client, question, "ffff:ffff:ffff:ffff::");
         }
     }
     
     while (!ok) {
-        debconf_input (client, "critical", "netcfg/get_netmask");
+        debconf_input (client, "critical", question);
         ret = debconf_go(client);
 
         if (ret)
             return ret;
 
-        debconf_get (client, "netcfg/get_netmask");
+        debconf_get (client, question);
 
         ptr = strtrim(client->value);
         ok = inet_pton (interface->address_family, ptr, &addr);
 
         if (!ok) {
             debconf_capb(client);
-            debconf_input (client, "critical", "netcfg/bad_ipaddress");
+            debconf_input (client, "critical", error);
             debconf_go (client);
             debconf_capb(client, "backup");
         }
@@ -117,8 +132,18 @@ static void netcfg_preseed_gateway(struct debconfclient *client,
                                    struct netcfg_interface *iface)
 {
     char ptr1[NETCFG_ADDRSTRLEN];
+    char *ptr;
     union inX_addr gw_addr, ipaddr, mask;
-    
+    const char *question;
+
+    switch (iface->address_family) {
+    case AF_INET6:
+        question="netcfg/get_ipv6_gateway";
+        break;
+    default:
+        question="netcfg/get_gateway";
+    }
+
     inet_pton(iface->address_family, iface->ipaddress, &ipaddr);
     inet_mton(iface->address_family, iface->masklen, &mask);
     
@@ -144,14 +169,16 @@ static void netcfg_preseed_gateway(struct debconfclient *client,
      * This won't *quite* work with anything shorter than a /24; such is life.
      */
     if (!strcmp(iface->ipaddress, ptr1)) {
-        char *ptr = strrchr(ptr1, iface->address_family == AF_INET6 ? ':' : '.');
+        ptr = strrchr(ptr1, iface->address_family == AF_INET6 ? ':' : '.');
         assert (ptr); /* if there's no separator in ptr1 we're in deep shit */
         ptr[1] = '\0';
     }
 
-    debconf_get(client, "netcfg/get_gateway");
-    if (empty_str(client->value))
-        debconf_set(client, "netcfg/get_gateway", ptr1);
+    debconf_get(client, question);
+    ptr = strtrim(client->value);
+    if (empty_str(ptr) ||
+        (strcmp(ptr, "none") && !inet_pton (iface->address_family, ptr, &gw_addr)))
+        debconf_set(client, question, ptr1);
 }
 
 static int netcfg_get_gateway(struct debconfclient *client, struct netcfg_interface *interface)
@@ -159,15 +186,30 @@ static int netcfg_get_gateway(struct debconfclient *client, struct netcfg_interf
     union inX_addr gw_addr;
     int ret, ok = 0;
     char *ptr;
+    const char *question, *error;
+
+    switch (interface->address_family) {
+    case AF_INET:
+        question="netcfg/get_gateway";
+        error="netcfg/bad_ipv4_address";
+        break;
+    case AF_INET6:
+        question="netcfg/get_ipv6_gateway";
+        error="netcfg/bad_ipv6_address";
+        break;
+    default:
+        question="netcfg/get_gateway";
+        error="netcfg/bad_ipaddress";
+    }
 
     while (!ok) {
-        debconf_input (client, "critical", "netcfg/get_gateway");
+        debconf_input (client, "critical", question);
         ret = debconf_go(client);
 
         if (ret)
             return ret;
 
-        debconf_get(client, "netcfg/get_gateway");
+        debconf_get(client, question);
         ptr = strtrim(client->value);
 
         if (empty_str(ptr) || /* No gateway, that's fine */
@@ -181,7 +223,7 @@ static int netcfg_get_gateway(struct debconfclient *client, struct netcfg_interf
 
         if (!ok) {
             debconf_capb(client);
-            debconf_input (client, "critical", "netcfg/bad_ipaddress");
+            debconf_input (client, "critical", error);
             debconf_go (client);
             debconf_capb(client, "backup");
         } else {
@@ -601,6 +643,11 @@ int netcfg_get_static(struct debconfclient *client, struct netcfg_interface *ifa
                 else if (iface->masklen == 0) {
                     state = GET_NETMASK;
                 } else {
+                    inet_mtop(iface->address_family, iface->masklen, netmask, NETCFG_ADDRSTRLEN);
+                    if (iface->address_family == AF_INET6)
+                        debconf_set(client, "netcfg/get_ipv6_netmask", netmask);
+                    else
+                        debconf_set(client, "netcfg/get_netmask", netmask);
                     state = GET_GATEWAY;
                 }
             }
@@ -643,7 +690,7 @@ int netcfg_get_static(struct debconfclient *client, struct netcfg_interface *ifa
             break;
         case GET_NAMESERVERS:
             if (nameservers) free(nameservers);
-            state = (netcfg_get_nameservers (client, &nameservers, iface->gateway)) ?
+            state = netcfg_get_nameservers (client, &nameservers, iface->gateway, iface) ?
                 GET_GATEWAY : CONFIRM;
             break;
         case GET_HOSTNAME:
